"""Constants package."""

from ani_scrapy.core.constants.general import (
    CONTEXT_OPTIONS,
    STAPE_DOWNLOAD_URL,
    YOURUPLOAD_DOWNLOAD_URL,
    YOURUPLOAD_TIMEOUT,
    SW_TIMEOUT,
    MEDIAFIRE_TIMEOUT,
    MONTH_MAP,
)

__all__ = [
    "CONTEXT_OPTIONS",
    "STAPE_DOWNLOAD_URL",
    "YOURUPLOAD_DOWNLOAD_URL",
    "YOURUPLOAD_TIMEOUT",
    "SW_TIMEOUT",
    "MEDIAFIRE_TIMEOUT",
    "MONTH_MAP",
]
