CONTEXT_OPTIONS = {
    "ignore_https_errors": True,
    "color_scheme": "dark",
    "locale": "es-EC",
    "no_viewport": True,
    "timezone_id": "America/Guayaquil",
    "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
    + "AppleWebKit/537.36 Chrome/120.0.0.0 Safari/537.36",
    "extra_http_headers": {
        "accept": "text/html,application/xhtml+xml,application/xml;"
        + "q=0.9,*/*;q=0.8",
        "accept-language": "es-EC,es;q=0.8,en-US;q=0.5,en;q=0.3",
        "accept-encoding": "gzip, deflate, br",
    },
    "viewport": {"width": 1080, "height": 720},
}


STAPE_DOWNLOAD_URL = "https://streamtape.com/get_video"
YOURUPLOAD_DOWNLOAD_URL = "https://yourupload.com/embed"

YOURUPLOAD_TIMEOUT = 10000
SW_TIMEOUT = 7000
MEDIAFIRE_TIMEOUT = 10000

MONTH_MAP = {
    "Enero": 1,
    "Febrero": 2,
    "Marzo": 3,
    "Abril": 4,
    "Mayo": 5,
    "Junio": 6,
    "Julio": 7,
    "Agosto": 8,
    "Septiembre": 9,
    "Octubre": 10,
    "Noviembre": 11,
    "Diciembre": 12,
}
